==> scatterbrainedsearch-0.2.0.crate: scatterbrainedsearch-0.2.0/src/lib.rs <==
/*
MIT License

Copyright (c) 2022 Molese

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/

#![deny(missing_docs, unsafe_code)]

//! Scatterbrained implementation of minigrep.

use std::{env, error::Error, fs::File, io::Read};

/// The necessary configurations for initializing scatterbrainedsearch.
///
pub struct Config {
    /// The file along with the path in which the pattern search will be conducted.
    pub file: String,
    /// The pattern to search for in the contents of the file.
    pub pattern: String,
}

impl Config {
    /// Initializes a new Config.
    ///
    /// Returns error if incorrect option is passed.
    pub fn new(mut args: env::Args) -> Result<Config, &'static str> {
        if args.len() > 3 {
            return Err("Expected 2 arguments");
        }

        /* Discard the first argument (args[0]), which is the filename along with path */
        args.next();

        /* arg0[1] & arg0[2] are file & pattern respectively. */
        let (file, pattern): (String, String) = (
            match args.next() {
                Some(arg) => arg,
                None => return Err("No file provided"),
            },
            match args.next() {
                Some(arg) => arg,
                None => return Err("No pattern provided"),
            },
        );

        /* var names are same as in the struct, so e don't need explicit key:val initialization style. */
        Ok(Config { file, pattern })
    }
}

/* The lifetime parameters says that the search results will be valid as long as the contents are valid.
 * Can last even after pattern goes out of scope. */
/// Returns the line(s) where the pattern is matched in the contents of the file.
pub fn search<'a>(pattern: &str, content: &'a str) -> Vec<&'a str> {
    let mut result: Vec<&str> = Vec::new();

    for line in content.lines() {
        if line.contains(pattern) {
            result.push(line);
        }
    }

    result
}

/// Returns summary which includes total matched occurence(s) of the pattern & line respectively.
pub fn summarize(occurrence: usize, line_count: usize) -> String {
    format!(
        "Matched: {} occurrence{} in {} line{}.",
        occurrence,
        if occurrence > 1 { "s" } else { "" },
        line_count,
        if line_count > 1 { "s" } else { "" }
    )
}

/// Runs the library logic.
/// Expects a reference to [`Config`](struct@Config).
///
/// # Errors
///
/// Returns error on the following situations:
///
/// - `file` doesn't exist.
/// - `file` could not be read (unauthorized read permission).
pub fn run(config: Config) -> Result<(), Box<dyn Error>> {
    /* `?` will return the error(ending fn execution)
     * if it encounters an `Err` in the `Result` it follows. */
    let (mut file, mut data): (File, String) = (File::open(&config.file)?, String::new());

    file.read_to_string(&mut data)?;

    let result: Vec<&str> = search(&config.pattern, &data);

    if result.is_empty() {
        println!("No match found for {} in {}", &config.pattern, &config.file);
    } else {
        let (mut line_count, mut total_occurrence): (usize, usize) = (0usize, 0usize);

        let mut matched_result: String = String::new();

        for line in result {
            let occurrence: usize = line.matches(&config.pattern).count();
            total_occurrence += occurrence;

            let formatted_output: String = format!("{} ~{}\n", line, occurrence);

            matched_result.push_str(&*formatted_output);
            line_count += 1;
        }

        let input_summary: String =
            format!("Lines matching {} in {}:", &config.pattern, &config.file);

        println!(
            "{}\n\n{}\n{}",
            input_summary,
            matched_result,
            summarize(total_occurrence, line_count)
        );
    }

    /* `()` is a unit type.
     * We do, however, care about the errors that might occur, and that's why the result type
     * exists with a dynamic error return type. */
    Ok(())
}

#[cfg(test)]
mod test {
    /* use every fn from above */
    use super::*;

    #[test]
    fn case_sensitive() {
        let pattern: &str = "omg";
        let data: &str = "omg, rust is so cool!";

        assert_eq!(vec![data], search(pattern, data));
    }

    #[test]
    fn case_sensitive_multiline() {
        let pattern: &str = "th";
        let data: &str = "Rust is the best.\nIt is the best.\nThe best!";

        assert_eq!(
            vec!["Rust is the best.", "It is the best."],
            search(pattern, data)
        );
    }

    #[test]
    fn case_insensitive() {
        let pattern: &str = "ok";
        let data: &str = "Ok, I'm going crazy!";

        assert_ne!(vec![data], search(pattern, data));
    }

    #[test]
    fn usual_summary() {
        assert_eq!("Matched: 1 occurrence in 1 line.", summarize(1, 1));
    }

    #[test]
    fn plural_summary() {
        assert_eq!("Matched: 2 occurrences in 2 lines.", summarize(2, 2));
    }

    #[test]
    fn plural_occurrence_summary() {
        assert_eq!("Matched: 2 occurrences in 1 line.", summarize(2, 1));
    }

    #[test]
    fn plural_line_count_summary() {
        assert_eq!("Matched: 1 occurrence in 2 lines.", summarize(1, 2));
    }
}

==> scatterbrainedsearch-0.3.0.crate: scatterbrainedsearch-0.3.0/src/lib.rs <==
/*
MIT License

Copyright (c) 2022 Molese

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/

#![deny(missing_docs, unsafe_code)]

//! Scatterbrained implementation of minigrep.

use std::{env, error::Error, fs::File, io::Read};

/// The necessary configurations for initializing scatterbrainedsearch.
///
pub struct Config {
    /// The file along with the path in which the pattern search will be conducted.
    pub file: String,
    /// The pattern to search for in the contents of the file.
    pub pattern: String,
}

impl Config {
    /// Initializes a new Config.
    ///
    /// Returns error if incorrect option is passed.
    pub fn new(mut args: env::Args) -> Result<Config, &'static str> {
        if args.len() > 3 {
            return Err("Expected 2 arguments");
        }

        /* Discard the first argument (args[0]), which is the filename along with path */
        args.next();

        /* arg0[1] & arg0[2] are file & pattern respectively. */
        let (file, pattern): (String, String) = (
            match args.next() {
                Some(arg) => arg,
                None => return Err("No file provided"),
            },
            match args.next() {
                Some(arg) => arg,
                None => return Err("No pattern provided"),
            },
        );

        /* var names are same as in the struct, so e don't need explicit key:val initialization style. */
        Ok(Config { file, pattern })
    }
}

/* The lifetime parameters says that the search results will be valid as long as the contents are valid.
 * Can last even after pattern goes out of scope. */
/// Returns the line(s) where the pattern is matched in the contents of the file.
pub fn search<'a>(pattern: &str, content: &'a str) -> Vec<&'a str> {
    let mut result: Vec<&str> = Vec::new();

    for line in content.lines() {
        if line.contains(pattern) {
            result.push(line);
        }
    }

    result
}

/// Returns summary which includes total matched occurence(s) of the pattern & line respectively.
pub fn summarize(occurrence: u128, line_count: u128) -> String {
    format!(
        "Matched: {} occurrence{} in {} line{}.",
        occurrence,
        if occurrence > 1 { "s" } else { "" },
        line_count,
        if line_count > 1 { "s" } else { "" }
    )
}

/// Runs the library logic.
/// Expects a reference to [`Config`](struct@Config).
///
/// # Errors
///
/// Returns error on the following situations:
///
/// - `file` doesn't exist.
/// - `file` could not be read (unauthorized read permission).
pub fn run(config: Config) -> Result<(), Box<dyn Error>> {
    /* `?` will return the error(ending fn execution)
     * if it encounters an `Err` in the `Result` it follows. */
    let (mut file, mut data): (File, String) = (File::open(&config.file)?, String::new());

    file.read_to_string(&mut data)?;

    let result: Vec<&str> = search(&config.pattern, &data);

    if result.is_empty() {
        println!("No match found for {} in {}", &config.pattern, &config.file);
    } else {
        let (mut line_count, mut total_occurrence): (u128, u128) = (0u128, 0u128);

        let mut matched_result: String = String::new();

        for line in result {
            let occurrence: u128 = line.matches(&config.pattern).count().try_into().unwrap();
            total_occurrence += occurrence;

            let formatted_output: String = format!("{} ~{}\n", line, occurrence);

            matched_result.push_str(&*formatted_output);
            line_count += 1;
        }

        let input_summary: String =
            format!("Lines matching {} in {}:", &config.pattern, &config.file);

        println!(
            "{}\n\n{}\n{}",
            input_summary,
            matched_result,
            summarize(total_occurrence, line_count)
        );
    }

    /* `()` is a unit type.
     * We do, however, care about the errors that might occur, and that's why the result type
     * exists with a dynamic error return type. */
    Ok(())
}

#[cfg(test)]
mod test {
    /* use every fn from above */
    use super::*;

    #[test]
    fn case_sensitive() {
        let pattern: &str = "omg";
        let data: &str = "omg, rust is so cool!";

        assert_eq!(vec![data], search(pattern, data));
    }

    #[test]
    fn case_sensitive_multiline() {
        let pattern: &str = "th";
        let data: &str = "Rust is the best.\nIt is the best.\nThe best!";

        assert_eq!(
            vec!["Rust is the best.", "It is the best."],
            search(pattern, data)
        );
    }

    #[test]
    fn case_insensitive() {
        let pattern: &str = "ok";
        let data: &str = "Ok, I'm going crazy!";

        assert_ne!(vec![data], search(pattern, data));
    }
}
